@@ -10,41 +10,58 @@
 #include <pybind11/operators.h>
 
 #include "odil/DataSet.h"
+#include "odil/VRFinder.h"
 
 #include "opaque_types.h"
 #include "type_casters.h"
 
+namespace
+{
+void 
+add_python_object(
+    std::shared_ptr<odil::DataSet> self, odil::Tag const & tag, 
+    pybind11::sequence source, odil::VR vr=odil::VR::UNKNOWN)
+{
+    if(vr == odil::VR::UNKNOWN)
+    {
+        static odil::VRFinder const vr_finder;
+        vr = vr_finder(tag, self, self->get_transfer_syntax());
+    }
+    if(pybind11::len(source) > 0)
+    {
+        self->add(tag, {convert_sequence<odil::Value>(source), vr});
+    }
+    else
+    {
+        self->add(tag);
+    }
+}
+}
+
 void wrap_DataSet(pybind11::module & m)
 {
     using namespace pybind11;
     using namespace odil;
 
     class_<DataSet, std::shared_ptr<DataSet>>(m, "DataSet")
-        .def(init<>())
-        .def(init<std::string const &>())
+        .def(
+            init([](std::string const & transfer_syntax, pybind11::kwargs kwargs)
+            {
+                auto self = std::make_shared<DataSet>(transfer_syntax);
+                for(auto && item: kwargs)
+                {
+                    add_python_object(
+                        self, item.first.cast<std::string>(), 
+                        item.second.cast<sequence>());
+                }
+                return self;
+            }),
+            "transfer_syntax"_a="")
         .def("add", (void (DataSet::*)(Tag const &, Element const &)) &DataSet::add)
         .def(
             "add", (void (DataSet::*)(Tag const &, VR)) &DataSet::add,
             "tag"_a, "vr"_a=VR::UNKNOWN)
-        .def(
-            "add",
-            [](DataSet & data_set, Tag const & tag, sequence source, VR vr)
-            {
-                if(vr == VR::UNKNOWN)
-                {
-                    vr = as_vr(tag);
-                }
-                if(len(source) > 0)
-                {
-                    data_set.add(tag, {convert_sequence<Value>(source), vr});
-                }
-                else
-                {
-                    data_set.add(tag);
-                }
-            },
-            "self"_a, "tag"_a, "vr"_a=VR::UNKNOWN
-        )
+        .def("add", add_python_object, "self"_a, "tag"_a, "vr"_a=VR::UNKNOWN)
         .def("remove", &DataSet::remove)
         .def("has", &DataSet::has)
         .def("empty", (bool (DataSet::*)() const) &DataSet::empty)
@@ -81,23 +98,41 @@ void wrap_DataSet(pybind11::module & m)
         .def("__len__", (std::size_t (DataSet::*)() const) &DataSet::size)
         .def(
             "__getitem__",
-            [](DataSet const & d, Tag const & t) { return d[t]; })
+            (Element & (DataSet::*)(Tag const &)) &DataSet::operator[],
+            return_value_policy::reference_internal)
         .def(
             "__setitem__",
-            [](DataSet & d, Tag const & t, Element const & e) { d[t] = e; })
+            [](DataSet & self, Tag const & t, Element const & e) { self[t] = e; })
+        .def("__delitem__", &DataSet::remove)
         .def(
             "__iter__",
-            [](DataSet const & d)
+            [](DataSet const & self)
             {
-                return make_key_iterator(d.begin(), d.end());
+                return make_key_iterator(self.begin(), self.end());
             },
             keep_alive<0, 1>())
+        .def(
+            "__contains__", 
+            [](DataSet const & self, Tag const & t) { return self.has(t); })
+        .def(
+            "get",
+            [](DataSet const & self, Tag const & tag, object default_) { 
+                if(self.has(tag))
+                {
+                    return cast(self[tag]);
+                }
+                else
+                {
+                    return default_;
+                }
+            },
+            "tag"_a, "default"_a=none())
         .def(
             "keys",
-            [](DataSet const & data_set)
+            [](DataSet const & self)
             {
                 list tags;
-                for(auto const & item: data_set)
+                for(auto const & item: self)
                 {
                     tags.append(item.first);
                 }
@@ -105,10 +140,10 @@ void wrap_DataSet(pybind11::module & m)
             })
         .def(
             "values",
-            [](DataSet const & data_set)
+            [](DataSet const & self)
             {
                 list tags;
-                for(auto const & item: data_set)
+                for(auto const & item: self)
                 {
                     tags.append(&item.second);
                 }
@@ -116,15 +151,50 @@ void wrap_DataSet(pybind11::module & m)
             })
         .def(
             "items",
-            [](DataSet const & data_set)
+            [](DataSet const & self)
             {
                 list tags;
-                for(auto const & item: data_set)
+                for(auto const & item: self)
                 {
                     tags.append(std::make_pair(item.first, &item.second));
                 }
                 return tags;
             })
+        .def(
+            "update",
+            [](DataSet & self, DataSet const & other) {
+                for(auto && item: other) 
+                {
+                    if(self.has(item.first)) 
+                    {
+                        self.remove(item.first);
+                    }
+                    self.add(item.first, item.second);
+                }
+            },
+            "other"_a)
+        .def(pickle(
+            [](DataSet const & data_set) {
+                tuple pickled(data_set.size());
+                int i = 0; 
+                for(auto && item: data_set)
+                {
+                    pickled[i] = make_tuple(item.first, item.second);
+                    ++i;
+                }
+                return pickled;
+            },
+            [](tuple pickled) {
+                DataSet data_set;
+                for(auto && item: pickled)
+                {
+                    data_set.add(
+                        item.cast<tuple>()[0].cast<Tag>(), 
+                        item.cast<tuple>()[1].cast<Element>());
+                }
+                return data_set;
+            }
+        ))
     ;
 
 }
